rm(list = ls())
#setwd("./")
library(stats)
library(MASS)
library(GeneNet)
source("./pcor_func.R")
TFs <- read.table("./3TF_genes_expression.txt", sep="\t", header = TRUE, row.names = 1)
TFs <- t(TFs)
TFs_names<-rownames(TFs)

for(k in 1:dim(TFs)[1])
{
    genes<-read.table("./TG2539.txt",skip=1, sep="\t", row.names = 1) 
    tfs<-TFs[k,]
    genes<-unique(genes)    ## unique genes
    gn<-rownames(genes) ##first col is names
    tfn<-TFs_names[k]  ##first col is names 
    
    print(paste("TF:", tfn, ", procedure:", k, "/", dim(TFs)[1]))
    
    Y<-tfs
    X<-genes #genes
    n=dim(X)[1]
    sig=0.15
    result=NULL
    for(i in 1:n)
    {
        x<-as.numeric(X[i,])
        y<-as.numeric(Y)   
        x<-(x-min(x))/(max(x)-min(x))
        x<-round(x,2)
        y<-(y-min(y))/(max(y)-min(y))
        y<-round(y,2)
        c1<-which(x<=.25)
        c2<-which(x>=.26&x<=.50)
        c3<-which(x>=.51&x<=.75)
        c4<-which(x>=.76&x<=1)
        x<-replace(x,c1,1)
        x<-replace(x,c2,2)
        x<-replace(x,c3,3)
        x<-replace(x,c4,4)
        
        d1<-which(y<=.25)
        d2<-which(y>=.26&y<=.50)
        d3<-which(y>=.51&y<=.75)
        d4<-which(y>=.76&y<=1)
        y<-replace(y,d1,1)
        y<-replace(y,d2,2)
        y<-replace(y,d3,3)
        y<-replace(y,d4,4)
        x<-factor(x,levels=c("1","2","3","4"))
        y<-factor(y,levels=c("1","2","3","4"))
        A<-table(x,y)      ### observed frequencies
        row_sum<-c(apply(A,1,sum))
        col_sum<-c(apply(A,2,sum))
        total_sum=sum(row_sum)
        E=matrix(1,nrow=nrow(A),ncol=ncol(A)) #### expected frequencies under null hypothesis----
        for(t in 1:nrow(A))
        {
            for(q in 1:ncol(A))
            {
                E[t,q]<-row_sum[t]*col_sum[q]/total_sum
            }
        }
        E=round(E,digits=2)  ### expected frequencies------------
        mod1<-"coexpressed"
        mod2<-"one level lagged"
        mod3<-"negatively coexpressed"
        mod4<-"two level lagged"
        mod<-c(mod1,mod2,mod3,mod4)
        fit<-fisher.test(A)
        p.value<-fit$p.value
        if(p.value>=sig)vec_1="independent"
        if(p.value<sig)
        {
            ##---------------------P(I)--coexpressed--------------------------
            obs1<-c(diag(A)) 
            exp1<-c(total_sum*(obs1/sum(obs1)))
            if(sum(obs1)==0){obs1.1<-rep(total_sum/length(obs1),length(obs1))
            exp1<-c(total_sum*(obs1.1/sum(obs1.1)))}
            s1<-(obs1-exp1)^2/exp1
            s1[is.nan(s1)]=0
            stat1<-sum(s1)
            pvalue1<-1-pchisq(stat1,df=(length(obs1)-1))
            
            
            ######--------------------P(II)--one -level lagged-----------------------
            obs2<-c(A[1,2],A[3,4],A[2,1],A[4,3])
            exp2<-c(total_sum*(obs2/sum(obs2)))
            if(sum(obs2)==0){obs2.1<-rep(total_sum/length(obs2),length(obs2))
            exp2<-c(total_sum*(obs2.1/sum(obs2.1)))}
            s2<-(obs2-exp2)^2/exp2
            s2[is.nan(s2)]=0
            stat2<-sum(s2)
            pvalue2<-1-pchisq(stat2,df=(length(obs2)-1))
            
            ####-------P(III)--negatively coexpressed -------------------------------
            
            obs3<-c(A[4,1],A[3,2],A[2,3],A[1,4])
            exp3<-c(total_sum*(obs3/sum(obs3)))
            if(sum(obs3)==0){obs3.1<-rep(total_sum/length(obs3),length(obs3))
            exp3<-c(total_sum*(obs3.1/sum(obs3.1)))}
            s3<-(obs3-exp3)^2/exp3
            s3[is.nan(s3)]=0
            stat3<-sum(s3)
            pvalue3<-1-pchisq(stat3,df=(length(obs3)-1))
            
            
            ####--------------P(IV)-two level lagged --------------------------
            obs4<-c(A[1,3],A[3,1],A[2,4],A[4,2])
            exp4<-c(total_sum*(obs4/sum(obs4)))
            if(sum(obs4)==0){obs4.1<-rep(total_sum/length(obs4),length(obs4))
            exp4<-c(total_sum*(obs4.1/sum(obs4.1)))}
            s4<-(obs4-exp4)^2/exp4
            s4[is.nan(s4)]=0
            stat4<-sum(s4)
            pvalue4<-1-pchisq(stat4,df=(length(obs4)-1))
            
            ##-------------------
            pvalue<-c(pvalue1,pvalue2,pvalue3,pvalue4)
            p<-which.max(pvalue)
            max_pvalue<-pvalue[p]
            if(max_pvalue>sig)vec_1=mod[p]
            if(max_pvalue<=sig)vec_1="independent"
        }
        if(p.value>=sig)pval<-0
        if(p.value<sig)pval<-max_pvalue
        out<-cbind(gn[i],tfn,p.value,pval,vec_1)
        colnames(out)<-c("gene","tf","pvalue_ind/dep","pvalue/type","relation")
        result<-rbind(result,out)
    }
    
    a <- "_Output_responsive"
    b <- tfn
    c <-".txt"
    fname<-paste(b,a, c, sep="")
    write.table(result,file=fname, sep="\t ",row.names=F)
    
    
    ##### ---- interference ------
    p3 <- BH_adjpvalue(as.numeric(result[,3]))
    pos <- which(p3 <= 0.05 & result[,4] > 0.25)
    if (length(pos) != 0){
        genes_responsive <- genes[pos,]
    } else {
        pos <- sort(p3, index.return = TRUE)$ix[1:200]
        genes_responsive <- genes[pos,]
    }
    
    gn_responsive<-rownames(genes_responsive)
    X <- genes_responsive
    n=dim(X)[1]
    sig=0.05     ## significance level to test the correlation and partial correlation , can be changed
    sig1=0.05     ## significance level to test the difference of correlations, can be changed
    sig2=0.05     ##significance level for causal direction
    sig_diff=0.15
    H<-matrix(0, nrow = n, ncol = n)
    result=NULL
    for(i in 1:(n-1))
    {
        for(j in (i+1):n)
        {
            x<-as.numeric(X[i,])
            y<-as.numeric(X[j,])
            z<-as.numeric(Y)
            fit<-cor.test(x,y, method="spearman", alternative="two.sided",exact=FALSE)
            r<-fit$estimate
            Cr<-cor(data.frame(x,y,z),method="spearman")
            pvalue<-fit$p.value 
            fit1<- pcor2.test(x, y, z, method =  "spearman") 
            pr <-fit1$estimate
            H[i,j]<-fit1$p.value
            pvalue1<-fit1$p.value                         ##pvalue for partial correlation
            cr<-Cr
            
            ## finding difference of correlations and pvalue of the test of significance of difference
            diff=abs(cr[1,2]-pr)
            if(diff<sig_diff){next}
            
            se<-sediff(cr=cr)
            if(se=="NaN")z1=0
            else
                z1<-diff/se
            pvalue3<-pnorm(-abs(z1)) ## pvalue for difference test
            ### case1
            #if(pvalue>sig&pvalue1[k]>sig)
            ### case1
            if (!is.na(pvalue) && !is.na(pvalue1[k]) && pvalue > sig && pvalue1[k] > sig)
                rel_vec <- "tf_not_intf"
            ##case 2
            rel_vec <- ifelse(
              !is.na(pvalue) && !is.na(pvalue1[k]),
              ifelse(pvalue > sig && pvalue1[k] < sig, "tf_intf", "tf_not_intf"),
              "tf_intf"
            )

#            if(pvalue>sig&pvalue1[k]<sig)
 #               rel_vec="tf_intf"
            ## Case 3
            rel_vec <- ifelse(
              !is.na(pvalue) && !is.na(pvalue1[k]) && pvalue < sig && pvalue1[k] < sig,
              ifelse(pvalue3 < sig1, "tf_intf", "tf_not_intf"),
              "tf_intf"
            )

  #        if(pvalue<sig&pvalue1[k]<sig)
   #             ## test for significant diff between correlations----
    #        {
    #            if(pvalue3<sig1){rel_vec="tf_intf"}
    #            if(pvalue3>=sig1){rel_vec="tf_not_intf"}
    #        }
            ##case 4
            if (!is.na(pvalue) && !is.na(pvalue1[k]) && pvalue < sig && pvalue1[k] > sig)
                rel_vec <- "tf_intf"
           # if(pvalue<sig&pvalue1[k]>sig)rel_vec="tf_intf"
            
            gene_names<-cbind(gn_responsive[i],gn_responsive[j],tfn)              ## names of genes pair and tf
            
            out<-cbind(gene_names,r,pvalue, pr, pvalue1,diff,pvalue3,rel_vec)
            colnames(out)<-c("pw_g1","pw_g2","tf","cor","cor_pval","pcor","pcor_pval","cor_diff","cor_diff_pval","Interference")
            #colnames(out)<-c("pw_g1","pw_g2","tf","cor_pval","pcor_pval","pcor","cor","cor_diff","diff_pval","rel_vec")
            result<-rbind(result,out)
        }
    }
    
    name_all <- paste0("./", tfn, "_Output_interference_all.txt")
    write.table(result,file=name_all,quote=F, sep="\t")
    result_1<-result[result[,10]=="tf_intf",]
    rownames(result_1)<-c(1:length(result_1[,1]))
    name_interferred <- paste0("./", tfn, "_Output_interference_interferred_only.txt")
    write.table(result_1,file=name_interferred,quote=F, sep="\t")
    
    # ------- organize results -----
    p_diff <- as.numeric(result_1[,"cor_diff_pval"])
    p_diff_adj <- BH_adjpvalue(p_diff)
    pos <- which(p_diff_adj <= 0.05)
    result_final <- result_1[pos,]
    
    long_result <- rbind(result_final[,-2], result_final[,-1])
    freq <- table(long_result[,1])
    a <- sort(freq, decreasing = TRUE)
    fname <- paste0("./", tfn, "_Output_interferred_frequency.txt")
    write.table(a,file=fname,quote=F, sep="\t",row.names = FALSE)
    
}








