 pcor2<-function (x, method = c("pearson", "kendall", "spearman"))
{
    method <- match.arg(method)
    if (is.data.frame(x))
        x <- as.matrix(x)
    if (!is.matrix(x))
        stop("supply a matrix-like 'x'")
    if (!(is.numeric(x) || is.logical(x)))
        stop("'x' must be numeric")
    stopifnot(is.atomic(x))
    n <- dim(x)[1]
    gp <- dim(x)[2] - 2
    cvx <- cov(x, method = method)
    icvx <- ginv(cvx)
    pcor <- -cov2cor(icvx)
    diag(pcor) <- 1
    if (method == "kendall") {
        statistic <- pcor/sqrt(2 * (2 * (n - gp) + 5)/(9 * (n -
            gp) * (n - 1 - gp)))
        p.value <- 2 * pnorm(-abs(statistic))
    }
    else {
        statistic <- pcor * sqrt((n - 2 - gp))/sqrt(round((1 - pcor^2),digits=10))
        p.value <- 2 * pnorm(-abs(statistic))
    }
    diag(statistic) <- 0
    diag(p.value) <- 0
    list(estimate = pcor, p.value = p.value, statistic = statistic,
        n = n, gp = gp, method = method)
}

 pcor2.test<-function (x, y, z, method = c("pearson", "kendall", "spearman"))
{
    method <- match.arg(method)
    x <- c(x)
    y <- c(y)
    z <- as.data.frame(z)
    xyz <- data.frame(x, y, z)
    pcor = pcor2(xyz, method = method)
    data.frame(estimate = pcor$est[1, 2], p.value = pcor$p.value[1,
        2], statistic = pcor$statistic[1, 2], n = pcor$n, gp = pcor$gp,
        Method = method)
}

BH_adjpvalue<-function(pvalues)
{
 ord<-order(pvalues,decreasing=TRUE)
 pval1<-sort(pvalues,decreasing=TRUE)
r1<-rank(pvalues,ties.method="first")
 len<-length(pval1)
 pval2<-c(1:len)
 pvalue<-c(1:len)
pvalue2<-c(1:len)
 for(i in 1:len)
 
 {
 pval2[i]=len/(len-(i-1))*pval1[i]
 }
 pvalue[ord]=pval2
 r2<-rank(pvalue,ties.method="first")
 for(j in 1:length(r1))
{
  b1<-which(r2==r1[j])
 pvalue2[j]<-pvalue[b1]
 }
 return(pvalue2)
 }
 



diff.corr.test <- function( r1, n1, r2, n2 ){ 

    Z1 <- 0.5 * log( (1+r1)/(1-r1) ) 
    Z2 <- 0.5 * log( (1+r2)/(1-r2) ) 

    diff   <- Z1 - Z2 
    SEdiff <- sqrt( 1/(n1 - 3) + 1/(n2 - 3) ) 
    diff.Z  <- diff/SEdiff 

    pval <- 2*pnorm( -abs(diff.Z), lower=F) 
    return(pval)
  } 


sediff<-function(cr)
{
   a=matrix(0,1,3)
   phi=matrix(0,3,3)
 a[1,1]=1-sqrt((1-cr[1,3]**2)*(1-cr[2,3]**2))
 a[1,2]=(cr[1,2]*cr[1,3]-cr[2,3])/(1-cr[1,3]**2)
 a[1,3]=(cr[1,2]*cr[1,3]-cr[1,3])/(1-cr[1,3]**2)
 
 phi[1,1]=(1-cr[1,2]**2)**2
 phi[2,2]=(1-cr[1,3]**2)**2
 phi[3,3]=(1-cr[2,3]**2)**2
 phi[1,2]=.5*(2*cr[2,3]-cr[1,2]*cr[1,3])*(1-cr[2,3]**2-cr[1,2]**2-cr[1,3]**2)+cr[2,3]**3
 phi[1,3]=.5*(2*cr[1,3]-cr[1,2]*cr[2,3])*(1-cr[2,3]**2-cr[1,2]**2-cr[1,3]**2)+cr[1,3]**3
 phi[2,3]=.5*(2*cr[1,2]-cr[1,3]*cr[2,3])*(1-cr[2,3]**2-cr[1,2]**2-cr[1,3]**2)+cr[1,2]**3
 phi[2,1]=phi[1,2]
 phi[3,1]=phi[1,3]
 phi[3,2]=phi[2,3]
 phi=phi/n
 sediff=sqrt((a%*%phi%*%t(a))/((1-cr[1,3]**2)*(1-cr[2,3]**2)))
 return(sediff)
 }
 


sediff1<-function(cr)
{
   a=matrix(0,1,3)
   phi=matrix(0,3,3)
 a[1,1]=1-1/(sqrt((1-cr[1,3]**2)*(1-cr[2,3]**2)))
 a[1,2]=(cr[1,3]-cr[1,2]*cr[2,3])/(sqrt(1-cr[1,3]**2)*(1-cr[2,3]**2)**(1.5))
 a[1,3]=(cr[2,3]-cr[1,2]*cr[1,3])/(sqrt(1-cr[2,3]**2)*(1-cr[1,3]**2)**(1.5))
 
 phi[1,1]=(1-cr[1,2]**2)**2
 phi[2,2]=(1-cr[2,3]**2)**2
 phi[3,3]=(1-cr[1,3]**2)**2
 phi[1,3]=.5*(2*cr[2,3]-cr[1,2]*cr[1,3])*(1-cr[2,3]**2-cr[1,2]**2-cr[1,3]**2)+cr[2,3]**3
 phi[1,2]=.5*(2*cr[1,3]-cr[1,2]*cr[2,3])*(1-cr[2,3]**2-cr[1,2]**2-cr[1,3]**2)+cr[1,3]**3
 phi[2,3]=.5*(2*cr[1,2]-cr[1,3]*cr[2,3])*(1-cr[2,3]**2-cr[1,2]**2-cr[1,3]**2)+cr[1,2]**3
 phi[2,1]=phi[1,2]
 phi[3,1]=phi[1,3]
 phi[3,2]=phi[2,3]
 phi=phi/n
 sediff=sqrt((a%*%phi%*%t(a)))
 return(sediff)
 }
 

### approach1 function---------------------

###------function for approach 1---------------------------
 approach1<-function(X,Y,gn,tfn)
{
n<-dim(X)[1]
m<-dim(Y)[1]
pr<-c(1:m)
pvalue1<-c(1:m)
   result=NULL
for(i in 1:(n-1))
{
 for(j in (i+1):n)
 {
 Cr<-array(0,c(3,3,m))
  for(k in 1:m)
 { 
   x<-as.numeric(X[i,])
   y<-as.numeric(X[j,])
   z<-as.numeric(Y[k,])
   fit<-cor.test(x,y, method="spearman", alternative="two.sided",exact=FALSE)
   r<-fit$estimate                                    ## correlation 
   Cr[,,k]<-cor(data.frame(x,y,z),method="spearman")
   pvalue<-fit$p.value                              ## pvalue for correlation
 fit1<- pcor2.test(x, y, z, method =  "spearman") 
 pr[k]<-fit1$estimate                               ## partial corr
  #H[i,j,k]<-fit1$p.value
 pvalue1[k]<-fit1$p.value                         ##pvalue for partial correlation
 cr<-Cr[,,k]
 
 ## finding difference of correlations and pvalue of the test of significance of difference
  
  diff=abs(cr[1,2]-pr[k])
  if(diff<sig_diff){next}
  se<-sediff(cr=cr)
   if(se=="NaN")z2=0
   else
    z2<-diff/se
   pvalue3<-pnorm(-abs(z2)) ## pvalue for difference test
 ### case1
  if(pvalue>sig&pvalue1[k]>sig)
  rel_vec<-"tf_not_intf"
 ##case 2
 if(pvalue>sig&pvalue1[k]<sig)
   rel_vec="tf_intf" 
   ## Case 3
 if(pvalue<sig&pvalue1[k]<sig)
 ## test for significant diff between correlations----
 {                 
   if(pvalue3<sig1){rel_vec="tf_intf"}
   if(pvalue3>=sig1){rel_vec="tf_not_intf"}
  }
  ##case 4
  if(pvalue<sig&pvalue1[k]>sig)rel_vec="tf_intf"
  ## regulator/no_regulator-------------
  if(abs(cr[1,2])>abs(pr[k]))vec_2="regulator"
  if(abs(cr[1,2])<=abs(pr[k]))vec_2="no_regulator"
  
  gene_names<-cbind(gn[i],gn[j],tfn[k])              ## names of genes pair and tf
  
out<-cbind(gene_names,pvalue,pvalue1[k],pr[k],r,diff,z2,pvalue3,rel_vec,vec_2)
colnames(out)<-c("pw_g1","pw_g2","tf","cor_pval(g1,g2)","pcor_pval(g1,g2|tf)","pcor","cor","cor_diff","diff_stat","dif_pval","rel_vec","regulator/no_regulator")
result<-rbind(result,out)
     }
  }
 }
 return(result)
}


#####--------------------------function for appr1-------------------------------------
###------------------pair cor----------------------------------------
pairCor <- function(x, gene = X) {
 fit<-cor.test(as.numeric(gene[x[1], ]), as.numeric(gene[x[2], ]),method="spearman", alternative="two.sided",exact=FALSE)
 corr<-fit$estimate
 pvalue<-fit$p.value
 return(list(corr,pvalue))
}
##-------------------------------------pcor func----------------------
pairPcor <- function(x, gene = X,tf=Y) {
 fit<-pcor2.test(as.numeric(gene[x[1], ]), as.numeric(gene[x[2], ]),as.numeric(tf[x[3], ]),method="spearman")
 pcorr<-fit$estimate
 pvalue1<-fit$p.value
 return(list(pcorr,pvalue1))
}

##--------------cor for data frame-----------------------
dfcor<-function(x,gene=X,tf=Y){    
Cr<-array(0,c(3,3,m))
Cr<-cor(data.frame(as.numeric(gene[x[1],]),as.numeric(gene[x[2],]),as.numeric(tf[x[3],])),method="spearman")
return(Cr)
}

###---------------function find diff results-------------------------------
diff_pval<-function(x,DFC=DF_cor,pr=pcorr){
  d<-abs(abs(DFC[,,x[1]][1,2])-abs(pr[x[1]]))
if(d<sig_diff)d_sig=0
 else {d_sig=1}
  se<-sediff(cr=DFC[,,x[1]])
   if(se=="NaN")z2=0
   else
    {z2<-d/se}
   pvalue3<-pnorm(-abs(z2))
 return(list(d,pvalue3,d_sig,z2))   
   }

##--------------------for mc_approach1--------------------------

diff_pval_1<-function(x,DFC=DF_cor,pr=pcorr,sig_diff1.1=sig_diff){
  d<-abs(abs(DFC[,,x[1]][1,2])-abs(pr[x[1]]))
if(d<sig_diff1.1)d_sig=0
 else {d_sig=1}
  se<-sediff(cr=DFC[,,x[1]])
   if(se=="NaN")z2=0
   else
    {z2<-d/se}
   pvalue3<-pnorm(-abs(z2))
 return(list(d,pvalue3,d_sig,z2))   
   }
 ###-------------function to find gene and tf names------------------------------  
 
  gn_names<-function(x,gene=gn,tf=tfn){
         names<-cbind(gene[x[1]],gene[x[2]],tf[x[3]])
		 return(names)
    }
	
####-------------------------------function compare cor pvalue and pcor pvalue---------
compare_pval<-function(x,value=pvalue,value1=pvalue1,value3=pvalue3,R=corr,pr=pcorr)
   {
     value<-c(rep(value, nrow(Y)))

     ### case1
  if(value[x[1]]>sig&value1[x[1]]>sig)
  rel_vec<-"tf_not_intf"
 ##case 2
 if(value[x[1]]>sig&value1[x[1]]<sig)
   rel_vec="tf_intf" 
   ## Case 3
 if(value[x[1]]<sig&value1[x[1]]<sig)
 ## test for significant diff between correlations----
 {                 
   if(value3[x[1]]<sig1){rel_vec="tf_intf"}
   if(value3[x[1]]>=sig1){rel_vec="tf_not_intf"}
  }
##case 4
  if(value[x[1]]<sig&value1[x[1]]>sig)rel_vec="tf_intf"
  ## regulator/no_regulator-------------
  if(abs(R[x[1]])>abs(pr[x[1]]))vec_2="regulator"
  if(abs(R[x[1]])<=abs(pr[x[1]]))vec_2="no_regulator"
   return(list(rel_vec,vec_2))
 }  

 ##-------------------------------------for mc_approach1----------------------------------------  
   compare_pval_1<-function(x,value=pvalue,value1=pvalue1,value3=pvalue3,R=corr,pr=pcorr,sig.1=sig,sig1.1=sig1)
   {
     value<-c(rep(value, nrow(Y)))

     ### case1
  if(value[x[1]]>sig.1&value1[x[1]]>sig.1)
  rel_vec<-"tf_not_intf"
 ##case 2
 if(value[x[1]]>sig.1&value1[x[1]]<sig.1)
   rel_vec="tf_intf" 
   ## Case 3
 if(value[x[1]]<sig.1&value1[x[1]]<sig.1)
 ## test for significant diff between correlations----
 {                 
   if(value3[x[1]]<sig1.1){rel_vec="tf_intf"}
   if(value3[x[1]]>=sig1.1){rel_vec="tf_not_intf"}
  }
##case 4
  if(value[x[1]]<sig.1&value1[x[1]]>sig.1)rel_vec="tf_intf"
  ## regulator/no_regulator-------------
  if(abs(R[x[1]])>abs(pr[x[1]]))vec_2="regulator"
  if(abs(R[x[1]])<=abs(pr[x[1]]))vec_2="no_regulator"
   return(list(rel_vec,vec_2))
 }  


approach2<-function(X,Y,gn,tfn,sig,sig_diff,sig1){
n<-dim(X)[1]
m<-dim(Y)[1]
##--------------------------------------------pair wise corr----------------------------
 pair<-combn(1:nrow(X), 2, simplify = F)  
 out <- mclapply(pair, pairCor) 
  out1<-unlist(out)
  v1<-c(seq(1,length(out1),2))
   corr<-out1[v1]                        ##------------------paiwise corr
   v2<-c(seq(2,length(out1),2))
    pvalue<-out1[v2]                     ##---------------pvalue for pairwise cor
##--------------------making triplet------------------------------
	pair<-combn(1:nrow(X), 2, simplify = F)
 pair2<-unlist(pair)
  v3<-cbind(pair2[seq(1,length(pair2),2)],pair2[seq(2,length(pair2),2)]) 
 T<-cbind(rep(v3[,1], nrow(Y)),rep(v3[,2], nrow(Y)),(rep(1:nrow(Y),each=length(pair))))
 T2<-apply(T,1,list)
 triple<-lapply(T2,unlist)
 ##----------------------------------finding partial cor using triplet--------------------

 out2 <- mclapply(triple, pairPcor)	
  out3<-unlist(out2)
  u1<-c(seq(1,length(out3),2))
   pcorr<-out3[u1]                    ##---partial cor
   u2<-c(seq(2,length(out3),2))
    pvalue1<-out3[u2]	            ##pvalue for partial corr
##---------------------------find pair aise cor for	triplet------------------------------

 df_cor <- mclapply(triple, dfcor)	      ## data frame correlation
DF_cor<-array(unlist(df_cor),c(3,3,length(df_cor)))
 ####---------------------------find triplet list----------------------------

          TL<- as.list(1:length(triple))
##-----------------------------cor diff results--------------------------------------
   diff_res<-mclapply(TL,diff_pval_1)
  df_res<-unlist(diff_res)
  w1<-c(seq(1,length(df_res),4))
   diff<-df_res[w1]                ## ---------diff
   w2<-c(seq(2,length(df_res),4))
    pvalue3<-df_res[w2]	            ##-------------pvalue for diff
	w3<-c(seq(3,length(df_res),4))
    d_sig<-df_res[w3]   ##-------------indicator for diff >diff_sig
	w4<-c(seq(4,length(df_res),4))
	z_scor<-df_res[w4]
##--------------------------finding-gene names ------------------------------------------
gene_names<-mclapply(triple,gn_names)
gnm<-unlist(gene_names)
 q1<-c(seq(1,length(gnm),3))  
   g1<-gnm[q1]                  ## --g1 name col1
   q2<-c(seq(2,length(gnm),3))
    g2<-gnm[q2]	                  ##---g2 name col2
	q3<-c(seq(3,length(gnm),3))
    g3<-gnm[q3]                   ##---tf name col3
##-------------------------comparing cor and pcor----------------------	
comp_res<-mclapply(TL,compare_pval_1)
comp_res2<-unlist(comp_res)
h1<-c(seq(1,length(comp_res2),2))
   rel_vec<-comp_res2[h1]                    ##---partial cor
   h2<-c(seq(2,length(comp_res2),2))
    vec<-comp_res2[h2]
cmb_res<-cbind(g1,g2,g3,pvalue,pvalue1,pcorr,corr,diff,z_scor,pvalue3,rel_vec,vec,d_sig)
cmb_res2<-cmb_res[d_sig==1,]
result<-cmb_res2[,-13]
colnames(result)<-c("pw_g1","pw_g2","tf","cor_pval(g1,g2)","pcor_pval(g1,g2|tf)","pcor","cor","cor_diff","diff_stat","dif_pval","rel_vec","regulator/no_regulator")
return(result)
}


mc_approach1<-function(X,Y,gn,tfn,sig,sig_diff,sig1){
n<-dim(X)[1]
m<-dim(Y)[1]

pair<-combn(1:nrow(X), 2, simplify = F) 
###------------------pair cor----------------------------------------

pairCor.1 <- function(x) {
 fit<-cor.test(as.numeric(X[x[1], ]), as.numeric(X[x[2], ]),method="spearman", alternative="two.sided",exact=FALSE)
 corr<-fit$estimate
 pvalue<-fit$p.value
 return(list(corr,pvalue))
}
 
 out <- mclapply(pair, pairCor.1) 
  out1<-unlist(out)
  v1<-c(seq(1,length(out1),2))
   corr<-out1[v1]                        ##------------------paiwise corr
   v2<-c(seq(2,length(out1),2))
    pvalue<-out1[v2]                     ##---------------pvalue for pairwise cor
##--------------------making triplet------------------------------
	pair<-combn(1:nrow(X), 2, simplify = F)
 pair2<-unlist(pair)
  v3<-cbind(pair2[seq(1,length(pair2),2)],pair2[seq(2,length(pair2),2)]) 
 T<-cbind(rep(v3[,1], nrow(Y)),rep(v3[,2], nrow(Y)),(rep(1:nrow(Y),each=length(pair))))
 T2<-apply(T,1,list)
 triple<-lapply(T2,unlist)
 ##----------------------------------finding partial cor using triplet--------------------
 ##-------------------------------------pcor func----------------------
pairPcor.1 <- function(x) {
 fit<-pcor2.test(as.numeric(X[x[1], ]), as.numeric(X[x[2], ]),as.numeric(Y[x[3], ]),method="spearman")
 pcorr<-fit$estimate
 pvalue1<-fit$p.value
 return(list(pcorr,pvalue1))
} 
 out2 <- mclapply(triple, pairPcor.1)	
  out3<-unlist(out2)
  u1<-c(seq(1,length(out3),2))
   pcorr<-out3[u1]                    ##---partial cor
   u2<-c(seq(2,length(out3),2))
    pvalue1<-out3[u2]	            ##pvalue for partial corr
##---------------------------find pair aise cor for	triplet------------------------------
##--------------cor for data frame-----------------------
dfcor.1<-function(x){    
Cr<-array(0,c(3,3,m))
Cr<-cor(data.frame(as.numeric(X[x[1],]),as.numeric(X[x[2],]),as.numeric(Y[x[3],])),method="spearman")
return(Cr)
}

 df_cor <- mclapply(triple, dfcor.1)	 ## data frame correlation
 DF_cor<-array(unlist(df_cor),c(3,3,length(df_cor)))
 ####---------------------------find triplet list----------------------------
          #TL<- as.list(1:length(triple))

##-----------------------------cor diff results--------------------------------------
  ###---------------function find diff results-------------------------------
diff_pval.1<-function(x){
  d<-abs(abs(DF_cor[,,x][1,2])-abs(pcorr[x]))
if(d<sig_diff)d_sig=0
 else {d_sig=1}
  se<-sediff(cr=DF_cor[,,x])
   if(se=="NaN")z2=0
   else
    {z2<-d/se}
   pvalue3<-pnorm(-abs(z2))
 return(list(d,pvalue3,d_sig,z2))   
   }

 diff_res<-mclapply(1:length(triple),diff_pval.1)
  df_res<-unlist(diff_res)
  w1<-c(seq(1,length(df_res),4))
   diff<-df_res[w1]                ## ---------diff
   w2<-c(seq(2,length(df_res),4))
    pvalue3<-df_res[w2]	            ##-------------pvalue for diff
	w3<-c(seq(3,length(df_res),4))
    d_sig<-df_res[w3]                  ##-------------indicator for diff >diff_sig
	w4<-c(seq(4,length(df_res),4))
	z_scor<-df_res[w4]
##--------------------------finding-gene names ------------------------------------------
###-------------function to find gene and tf names------------------------------  
 
  gn_names<-function(x){
         names<-cbind(gn[x[1]],gn[x[2]],tfn[x[3]])
		 return(names)
    }

gene_names<-mclapply(triple,gn_names)
gnm<-unlist(gene_names)
 q1<-c(seq(1,length(gnm),3))  
   g1<-gnm[q1]                  ## --g1 name col1
   q2<-c(seq(2,length(gnm),3))
    g2<-gnm[q2]	                  ##---g2 name col2
	q3<-c(seq(3,length(gnm),3))
    g3<-gnm[q3]                   ##---tf name col3
##-------------------------comparing cor and pcor----------------------
compare_pval.1<-function(x)
   {
   pvalue<-c(rep(pvalue, nrow(Y)))
   corr<-c(rep(corr,nrow(Y)))
     ### case1
  if(pvalue[x]>sig&pvalue1[x]>sig)
  rel_vec<-"tf_not_intf"
 ##case 2
 if(pvalue[x]>sig&pvalue1[x]<sig)
   rel_vec="tf_intf" 
   ## Case 3
 if(pvalue[x]<sig&pvalue1[x]<sig)
 ## test for significant diff between correlations----
 {                 
   if(pvalue3[x]<sig1){rel_vec="tf_intf"}
   if(pvalue3[x]>=sig1){rel_vec="tf_not_intf"}
  }
##case 4
  if(pvalue[x]<sig&pvalue1[x]>sig)rel_vec="tf_intf"
  ## regulator/no_regulator-------------
  if(abs(corr[x])>abs(pcorr[x]))vec_2="regulator"
  if(abs(corr[x])<=abs(pcorr[x]))vec_2="no_regulator"
   return(list(rel_vec,vec_2))
 }  
	
comp_res<-mclapply(1:length(triple),compare_pval.1)
comp_res2<-unlist(comp_res)
h1<-c(seq(1,length(comp_res2),2))
   rel_vec<-comp_res2[h1]                    ##---partial cor
   h2<-c(seq(2,length(comp_res2),2))
    vec<-comp_res2[h2]
cmb_res<-cbind(g1,g2,g3,pvalue,pvalue1,pcorr,corr,diff,z_scor,pvalue3,rel_vec,vec,d_sig)
cmb_res2<-cmb_res[d_sig==1,]
result<-cmb_res2[,-13]
colnames(result)<-c("pw_g1","pw_g2","tf","cor_pval(g1,g2)","pcor_pval(g1,g2|tf)","pcor","cor","cor_diff","diff_stat","dif_pval","rel_vec","regulator/no_regulator")
return(result)
}
